# $URL: //local/member/autrijus/Parse-AFP/lib/Parse/AFP/BDD.pm $ $Author: autrijus $
# $Rev: 1130 $ $Date: 2004-02-17T15:40:29.640821Z $

package Parse::AFP::BDD;
use base 'Parse::AFP::Record';

use constant SUBFORMAT => (
    Data => 'a*',    # not yet parsed!
);

1;
