# $URL: //local/member/autrijus/Parse-AFP/lib/Parse/AFP/PGP1.pm $ $Author: autrijus $
# $Rev: 1133 $ $Date: 2004-02-17T20:32:41.918436Z $

package Parse::AFP::PGP1;
use base 'Parse::AFP::Record';

use constant SUBFORMAT => (
    Data => 'a*',    # not yet parsed!
);

1;
